/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

#include <common.h>
//#include <asm/arch/drv_display.h>
#include <sys_config.h>
#include <asm/arch/timer.h>
#include <power/sunxi/pmu.h>
#include <power/sunxi/power.h>
#include <sunxi_board.h>
#include <fdt_support.h>
#include <sys_config_old.h>

DECLARE_GLOBAL_DATA_PTR;

typedef enum __BOOT_POWER_STATE
{
	STATE_SHUTDOWN_DIRECTLY = 0,
	STATE_SHUTDOWN_CHARGE,
	STATE_ANDROID_CHARGE,
	STATE_NORMAL_BOOT
}BOOT_POWER_STATE_E;

//power_start
//0:  not allow boot by insert dcin: press power key in long time & pre state is system state(Battery ratio shouled enough)
//1: allow boot directly  by insert dcin:( Battery ratio shouled enough)
//2: not allow boot by insert dcin:press power key in long time & pre state is system state(do not check battery ratio)
//3: allow boot directly  by insert dcin:( do not check battery ratio)
uint32_t PowerStart = 0;

int __sunxi_bmp_display(char* name)
{
	return 0;
}


int sunxi_bmp_display(char * name)
	__attribute__((weak, alias("__sunxi_bmp_display")));



static void UpdateChargeVariable(void)
{
	#if 0
	int ChargeMode = 0;
	if(0 == script_parser_fetch("charging_type", "charging_type", &ChargeMode, 1))
	{
		gd->chargemode = 1;
	}
	#endif
	//the default mode is use Android charge
	gd->chargemode = 1;
}


static void EnterNormalShutDownMode(void)
{
	sunxi_board_shutdown();
	for(;;);
}

static void EnterLowPowerShutDownMode(void)
{

	printf("battery ratio is low without  dc or ac, should be ShowDown\n");
#if defined(CONFIG_SUN8IW12P1_NOR) || defined(CONFIG_SUN8IW16P1_NOR)
	/*use pmu led and 2Hz frequency*/
	axp_set_led_control(0x02);
	__msdelay(3000);
	/*close pmu led */
	axp_set_led_control(0x00);
	__msdelay(1000);
#else
	#ifdef CONFIG_EINK_PANEL_USED
	board_display_eink_update("bat\\low_pwr.bmp", 0x04);
	#else
	#if defined(CONFIG_SUNXI_DISPLAY)
	sunxi_bmp_display("bat\\low_pwr.bmp");
	#endif
	#endif
	__msdelay(3000);
#endif
	sunxi_board_shutdown();
	for(;;);
}

static void EnterShutDownWithChargeMode(void)
{
	printf("battery low power and vol with dc or ac, should charge longer\n");
	#ifdef CONFIG_EINK_PANEL_USED
	board_display_eink_update("bat\\bempty.bmp", 0x04);
	#else
	#if defined(CONFIG_SUNXI_DISPLAY)
	sunxi_bmp_display("bat\\bempty.bmp");
	#endif
	#endif
	__msdelay(3000);
	sunxi_board_shutdown();
	for(;;);
}

static void EnterAndroidChargeMode(void)
{
#if defined(CONFIG_SUN8IW12P1_NOR) || defined(CONFIG_SUN8IW16P1_NOR) || defined(CONFIG_SUN8IW16P1_AXP2101_NOR)
	printf("startup system with ac or vubs\n");
	int ret = -1;
#if defined(CONFIG_SUNXI_LOAD_JPEG)
	ret = sunxi_load_jpeg("bootlogo.jpg");
#endif
	if (ret)
		ret = read_bmp_to_kernel("bootlogo");
#else
	printf("sunxi_bmp_charger_display\n");
	#ifdef CONFIG_EINK_PANEL_USED
	board_display_eink_update("bat\\battery_charge.bmp", 0x04);
	#else
	#if defined(CONFIG_SUNXI_DISPLAY)
	sunxi_bmp_display("bat\\battery_charge.bmp");
	#endif
	#endif
#endif

	UpdateChargeVariable();
}

static void EnterNormalBootMode(void)
{
	printf("sunxi_bmp_logo_display\n");
	#ifdef CONFIG_EINK_PANEL_USED
	board_display_eink_update("bootlogo.bmp", 0x04);
	#else
#if defined(CONFIG_SUN8IW12P1_NOR) || defined(CONFIG_SUN8IW16P1_NOR) || defined(CONFIG_SUN8IW16P1_AXP2101_NOR)
	int ret = -1;
#if defined(CONFIG_SUNXI_LOAD_JPEG)
	ret = sunxi_load_jpeg("bootlogo.jpg");
#endif
	if (ret)
		ret = read_bmp_to_kernel("bootlogo");
#else
	#if defined(CONFIG_SUNXI_DISPLAY)
	sunxi_bmp_display("bootlogo.bmp");
	#endif
#endif
	#endif
}



int ProbePreSystemMode(void)
{
    int  PreSysMode = 0;

    PreSysMode = axp_probe_pre_sys_mode();
    if(PreSysMode == PMU_PRE_SYS_MODE )
    {
        printf("pre sys mode\n");
        return PMU_PRE_SYS_MODE;
    }
    else if(PreSysMode == PMU_PRE_CHARGE_MODE)
    {
        printf("pre charge mode\n");
        return PMU_PRE_CHARGE_MODE;
    }
    else if(PreSysMode == PMU_PRE_FASTBOOT_MODE)
    {
        printf("pre fastboot mode\n");
        return PMU_PRE_FASTBOOT_MODE;
    }

    return 0;
}

static int ProbeStartupCause(void)
{
	uint PowerOnCause = 0;
	PowerOnCause = axp_probe_startup_cause();
	if(PowerOnCause == AXP_POWER_ON_BY_POWER_KEY)
	{
		printf("key trigger\n");
	}
	else if(PowerOnCause == AXP_POWER_ON_BY_POWER_TRIGGER)
	{
		printf("power trigger\n");
	}
	return PowerOnCause;
}


//check battery and voltage
static int GetBatteryRatio( void)
{
	int  Ratio ;

	Ratio = axp_probe_rest_battery_capacity();
	if(Ratio < 1)
	{
		//some board coulombmeter value is not precise whit low capacity, so open it again here
		//note :in this case ,you should wait at least 1s berfore you read battery ratio again 
		axp_set_coulombmeter_onoff(0);
		axp_set_coulombmeter_onoff(1);
	}
	return Ratio;
}

/*
*function : GetStateOnLowBatteryRatio
*@PowerBus           :   0: power  not exist   other:  power  exist
*@LowVoltageFlag :   0:high voltage  1: low voltage
*@PowerOnCause  :   Power is inserted OR   power_key is pressed
*note:  Decide which state should enter when battery ratio is low
*/
static BOOT_POWER_STATE_E GetStateOnLowBatteryRatio(int PowerBus,int LowVoltageFlag,int PowerOnCause)
{
	BOOT_POWER_STATE_E BootPowerState;

	do {
		//power  not exist,shutdown directly
		if(PowerBus == 0)
		{
			BootPowerState = STATE_SHUTDOWN_DIRECTLY;
			break;
		}

		//----------------power  exist: dcin or vbus------------------
		//user config is 3, allow boot directly  by insert dcin, not check battery ratio
		if(PowerStart == 3)
		{
			BootPowerState = STATE_NORMAL_BOOT;
			break;
		}

#if defined(CONFIG_SUN8IW12P1_NOR) || defined(CONFIG_SUN8IW16P1_NOR) || defined(CONFIG_SUN8IW16P1_AXP2101_NOR)
		/*sdv product: you should startup when ac or vbus exist*/
		if (LowVoltageFlag) {
			/*checking exist or not*/
			if (PowerBus != 0)
				BootPowerState = STATE_NORMAL_BOOT;
			else
				BootPowerState = STATE_SHUTDOWN_DIRECTLY;
		} else { /*high voltage*/
			/*checking acin exist ,open it*/
			if (PowerBus != 0)
				BootPowerState = STATE_ANDROID_CHARGE;
			else
				BootPowerState = STATE_NORMAL_BOOT;
		}
#else
		/*low voltage*/
		if (LowVoltageFlag) {
			BootPowerState = STATE_SHUTDOWN_CHARGE;
		} else { /*high voltage*/
			BootPowerState =
			    (PowerOnCause == AXP_POWER_ON_BY_POWER_TRIGGER) ? \
				    STATE_ANDROID_CHARGE:STATE_SHUTDOWN_CHARGE;
		}
#endif
	}while(0);
	return BootPowerState;
}

static BOOT_POWER_STATE_E GetStateOnHighBatteryRatio(int PowerBus,int LowVoltageFlag,int PowerOnCause)
{
	BOOT_POWER_STATE_E BootPowerState;
	//battery ratio enougth
	//note : 
	//enter android charge:  power is  inserted and PowerStart not allow boot directly  by insert dcin
	//enter normal  boot:  other way

	BootPowerState = STATE_NORMAL_BOOT;
	if(PowerOnCause == AXP_POWER_ON_BY_POWER_TRIGGER)
	{
		//user config is 3 or 1, allow boot directly  by insert dcin,
		if(PowerStart == 3 || PowerStart == 1)
		{
			BootPowerState = STATE_NORMAL_BOOT;
		}
		else
		{
			BootPowerState = STATE_ANDROID_CHARGE;
		}
	}

	return BootPowerState;
}

static void GetBatVolCap(int *BatVol, int *BatRatio)
{
	int count = 5;
	*BatVol = axp_probe_battery_vol();
	*BatRatio = GetBatteryRatio();
	while(count > 0 && *BatRatio == 100 && *BatVol < 4000){
	/* if ratio=100 & vol < 4v, it means a wrong ratio,detect again! */
		printf("msdelay 300ms!\n");
		__msdelay(300);
		printf("detect bat ratio again!\n");
		*BatRatio = GetBatteryRatio();
		count--;
	}
}

//function : PowerCheck
//para: null
//note:  Decide whether to boot
int PowerCheck(void)
{
	int BatExist = 0;
	int PowerBus=0;
	int BatVol=0;
	int BatRatio=0;
	int SafeVol =0;
	int PowerOnCause = 0;
	int Ret = 0;
	int LowVoltageFlag = 0;
	int LowBatRatioFlag =0;
	BOOT_POWER_STATE_E BootPowerState;
	int nodeoffset;
	uint pmu_bat_unused = 0;

	if(get_boot_work_mode() != WORK_MODE_BOOT)
	{
		return 0;
	}

	nodeoffset =  fdt_path_offset(working_fdt,PMU_SCRIPT_NAME);
	if(nodeoffset >0)
	{
		script_parser_fetch(PMU_SCRIPT_NAME, "power_start", (int *)&PowerStart, 1);
		script_parser_fetch(PMU_SCRIPT_NAME, "pmu_bat_unused", (int *)&pmu_bat_unused, 1);
	}
	//clear  power key 
	axp_probe_key();

	//check battery
	BatExist = pmu_bat_unused?0:axp_probe_battery_exist();

	//check power bus
	PowerBus = axp_probe_power_source();
	printf("PowerBus = %d( %d:vBus %d:acBus other: not exist)\n", PowerBus,AXP_VBUS_EXIST,AXP_DCIN_EXIST);

	power_limit_for_vbus(BatExist,PowerBus);

	if(BatExist <= 0)
	{
		printf("no battery exist\n");
		EnterNormalBootMode();
		return 0;
	}

	//if android call shutdown when  charing , then boot should enter android charge mode
	if((PMU_PRE_CHARGE_MODE == ProbePreSystemMode()))
	{
		if(PowerBus)
		{
			EnterAndroidChargeMode();
		}
		else
		{
			printf("pre system is charge mode,but without dc or ac, should be ShowDown\n");
			EnterNormalShutDownMode();
		}
		return 0;
	}

	//check battery ratio
	GetBatVolCap(&BatVol, &BatRatio);
	printf("Battery Voltage=%d, Ratio=%d\n", BatVol, BatRatio);

	//PMU_SUPPLY_DCDC2 is for cpua
	Ret = script_parser_fetch(PMU_SCRIPT_NAME, "pmu_safe_vol", &SafeVol, 1);
	if((Ret) || (SafeVol < 3000))
	{
		SafeVol = 3500;
	}
	LowBatRatioFlag =  (BatRatio<1) ? 1:0;
	LowVoltageFlag  =  (BatVol<SafeVol) ? 1:0;
	PowerOnCause = ProbeStartupCause();
	#ifdef	CONFIG_SUNXI_AXP2585
	if (LowBatRatioFlag || LowVoltageFlag)
	#else
	if(LowBatRatioFlag)
	#endif
	{
		//low battery ratio
		BootPowerState = GetStateOnLowBatteryRatio(PowerBus,LowVoltageFlag,PowerOnCause);
	}
	else
	{
		//high battery ratio
		BootPowerState = GetStateOnHighBatteryRatio(PowerBus,LowVoltageFlag,PowerOnCause);
	}

	switch(BootPowerState)
	{
		case STATE_SHUTDOWN_DIRECTLY:
			EnterLowPowerShutDownMode();
			break;
		case STATE_SHUTDOWN_CHARGE:
			EnterShutDownWithChargeMode();
			break;
		case STATE_ANDROID_CHARGE:
			EnterAndroidChargeMode();
			break;
		case STATE_NORMAL_BOOT:
			EnterNormalBootMode();
			break;
		default:
			printf("%s: error boot mode\n",__func__);
			break;
	}
	return 0;

}
